@@ -1,5 +1,5 @@
 # to avoid 'no visible binding for global variable' NOTE
-globalVariables(c(".", "un", "directory", "base64_enc", "server"))
+globalVariables(c(".", "un", "directory", "base64_enc", "server", "team"))
 
 #' gitgadget
 #'
@@ -11,6 +11,6 @@ globalVariables(c(".", "un", "directory", "base64_enc", "server"))
 #' @importFrom markdown markdownToHTML
 #' @importFrom utils read.csv capture.output
 #' @importFrom stats na.omit setNames
-#' @importFrom utils packageVersion browseURL
+#' @importFrom utils packageVersion browseURL head
 #' @importFrom usethis edit_r_environ edit_r_profile use_git use_github
 NULL
